using System.Collections.Generic;
using Halfling.Serialization;
using Halfling.TreeData;
using TankyTank.Game.Tanks;
using TankyTank.Modes;
using IGame=TankyTank.Game.IGame;

namespace TankyTank.AIBots
{
	/// <summary>
	/// A move/attack behavior that causes the tank to engage in combat with enemy tanks carrying friendly flags.
	/// </summary>
	public class AttackEnemyWithFlagBehavior : AttackEnemyBehavior
	{
		#region Constructors

		/// <summary>
		/// Loads a new AttackEnemyBehavior.
		/// </summary>
		public AttackEnemyWithFlagBehavior(IGame game, INode node, Serializer ser)
			: base(game, node, ser)
		{
			// Do nothing extra.
		}

		/// <summary>
		/// Creates a new AttackEnemyBehavior.
		/// </summary>
		public AttackEnemyWithFlagBehavior(IGame game, Settings settings)
			: base(game, settings)
		{
			// Do nothing extra.
		}

		#endregion
		#region Non-Public Methods

		/// <summary>
		/// Returns whether this behavior is currently valid.
		/// </summary>
		protected override bool IsValidBehavior()
		{
			if(!base.IsValidBehavior())
				return false;
			if(!(_game.ModeInfo is ICTFInfo))
				return false;
			return true;
		}

		/// <summary>
		/// Returns whether the current target should be cleared under the current conditions.
		/// </summary>
		/// <returns>True if the target was cleared.</returns>
		protected override bool ShouldClearTarget()
		{
			ICTFInfo ctfInfo = (ICTFInfo)_game.ModeInfo;
			return _target != null &&
			       (!ctfInfo.IsTankCarryingFlag(_target) ||
			        !AITools.Exists(_target, _game) ||
			        AITools.AreAllied(_curTank, _target));
		}

		/// <summary>
		/// Searches for possible tanks to attack.
		/// </summary>
		protected override IEnumerable<ITank> GetValidTargets()
		{
			ICTFInfo ctfInfo = (ICTFInfo)_game.ModeInfo;
			return ctfInfo.GetTanksWithFlags(_curTank.Team);
		}

		#endregion
	}
}