﻿using System;
using System.Reflection;
using Halfling;
using Halfling.Serialization;
using Halfling.TreeData;
using TankyTank.Game;
using TD = Halfling.TreeData;

namespace TankyTank.AIBots
{
	/// <summary>
	/// A behavior-based bot that deserializes its behavior from its tree file.
	/// </summary>
	public class CustomBehaviorBot : BehaviorBotBase
	{
		#region Constructors

		/// <summary>
		/// Creates a new CustomBehaviorBot.
		/// </summary>
		/// <param name="game">The game currently being played.</param>
		/// <param name="node">The node from which this bot was loaded.</param>
		/// <param name="ser">Serializer used to deserialize settings.</param>
		public CustomBehaviorBot(IGame game, INode node, Serializer ser)
			: base(game, node, ser)
		{
			//Parent behaviorsList = (Parent)((Parent)node)["Behaviors"];
			Parent behaviorsList = (Parent)((Parent)node).FindAtPath("Behaviors");
			foreach(Group behaviorGroup in behaviorsList)
			{
				string dll = ser.TreeDeserialize<string>(behaviorGroup, "DLL");
				string classname = ser.TreeDeserialize<string>(behaviorGroup, "Class");
				Assembly a = Assembly.LoadFile(new AbsolutePath(node.FileRoot.FilePath.Directory, dll));
				Type t = a.GetType(classname);
				ConstructorInfo ci = t.GetConstructor(new[] { typeof(IGame), typeof(INode), typeof(Serializer) });
				object behavior = ci.Invoke(new object[] { game, behaviorGroup, ser });
				if(behavior is IMoveAttackBehavior)
					AddBehavior((IMoveAttackBehavior)behavior);
				else if(behavior is IMoveBehavior)
					AddBehavior((IMoveBehavior)behavior);
				else if(behavior is IAttackBehavior)
					AddBehavior((IAttackBehavior)behavior);
				else
					throw new NotSupportedException("Behavior type '" + behavior.GetType() + "' is not supported.");
			}
		}

		/// <summary>
		/// Creates a new CustomBehaviorBot.
		/// </summary>
		/// <param name="game">The game currently being played.</param>
		/// <param name="file">The txt file from which this bot was loaded.</param>
		/// <param name="ser">Serializer used to deserialize settings.</param>
		public CustomBehaviorBot(IGame game, TreeFile file, Serializer ser)
			: this(game, (INode)file, ser)
		{
			// Do nothing extra.
		}

		/// <summary>
		/// Creates a new CustomBehaviorBot.
		/// </summary>
		/// <param name="game">The game currently being played.</param>
		/// <param name="settings">The bot settings.</param>
		public CustomBehaviorBot(IGame game, Settings settings)
			: base(game, settings)
		{
			// Do nothing extra.
		}

		#endregion
	}
}