using System.Collections.Generic;
using Halfling.Serialization;
using Halfling.TreeData;
using TankyTank.Game.Doodads;
using TankyTank.Modes;
using IGame=TankyTank.Game.IGame;

namespace TankyTank.AIBots
{
	/// <summary>
	/// A move behavior that causes the tank to rescue captured-then-dropped friendly flags.
	/// </summary>
	public class RescueFlagBehavior : CollectPowerupBehavior
	{
		#region Constructors

		/// <summary>
		/// Loads a new RescueFlagBehavior.
		/// </summary>
		public RescueFlagBehavior(IGame game, INode node, Serializer ser)
			: base(game, node, ser)
		{
			// Do nothing extra.
		}

		/// <summary>
		/// Creates a new RescureFlagBehavior.
		/// </summary>
		public RescueFlagBehavior(IGame game, Settings settings)
			: base(game, settings)
		{
			// Do nothing extra.
		}

		#endregion
		#region Non-Public Methods

		/// <summary>
		/// Returns whether this behavior is currently valid.
		/// </summary>
		protected override bool IsValidBehavior()
		{
			if(!base.IsValidBehavior())
				return false;
			if(!(_game.ModeInfo is ICTFInfo))
				return false;
			ICTFInfo modeInfo = (ICTFInfo)_game.ModeInfo;
			if(modeInfo.IsTankCarryingFlag(_curTank))
				return false;
			return true;
		}

		/// <summary>
		/// Returns whether the current target should be cleared under the current circumstances.
		/// </summary>
		protected override bool ShouldClearTarget()
		{
			return _target != null &&
				   !AITools.Exists(_target, _game);
		}

		/// <summary>
		/// Returns the valid targets from which the target can be chosen.
		/// </summary>
		protected override IEnumerable<IDoodad> GetValidTargets()
		{
			ICTFInfo ctfInfo = (ICTFInfo)_game.ModeInfo;
			return ctfInfo.GetRescuableFlags(_curTank.Team);
		}

		#endregion
	}
}