﻿using System.Collections.Generic;
using TankyTank.Game.Doodads;
using TankyTank.Game.Tanks;

namespace TankyTank.Modes
{
	/// <summary>
	/// Exposes methods and properties that allow information about a capture-the-flag game to be retrieved.
	/// </summary>
	public interface ICTFInfo
	{
		#region Methods

		/// <summary>
		/// Returns the flag doodads.
		/// </summary>
		/// <param name="flagTeam">The team to which the flags belong.</param>
		IEnumerable<IDoodad> GetFlags(int flagTeam);

		/// <summary>
		/// Returns the flag doodads that have been not been captured or moved by the enemy.
		/// </summary>
		/// <param name="flagTeam">The team to which the flags belong.</param>
		IEnumerable<IDoodad> GetUncapturedFlags(int flagTeam);

		/// <summary>
		/// Returns the flag doodads that were captured by an enemy who was then destroyed
		/// and thus the flag needs to be rescued.
		/// </summary>
		/// <param name="flagTeam">The team to which the flags belong.</param>
		IEnumerable<IDoodad> GetRescuableFlags(int flagTeam);

		/// <summary>
		/// Returns the tanks currently carrying flags.
		/// </summary>
		/// <param name="flagTeam">The team to which the flags belong.</param>
		IEnumerable<ITank> GetTanksWithFlags(int flagTeam);

		/// <summary>
		/// Returns whether the specified tank is currently carrying a flag.
		/// </summary>
		bool IsTankCarryingFlag(ITank tank);

		/// <summary>
		/// Returns the flag return point doodads to which captured flags must be delivered.
		/// </summary>
		/// <param name="rpTeam">The team to which the returned flag return points belong.</param>
		IEnumerable<IDoodad> GetReturnPoints(int rpTeam);

		#endregion
	}
}