// Walt Destler

using System.Collections.Generic;
using System.Drawing;
using Halfling.Geometry;
using Halfling.Geometry.Dim2;
using Halfling.Serialization;
using Halfling.TreeData;
using TankyTank.AI;
using TankyTank.AIBots;
using TankyTank.Game;
using TankyTank.Game.Doodads;
using TankyTank.Game.Tanks;
using TankyTank.Modes;

namespace TankyTank.Scenarios
{
	/// <summary>
	/// Raid On Metropolis server _game mode.
	/// </summary>
	public class ROMServer : CTFServer
	{
		#region Private Fields

		private readonly INode aiNode;
		private int victoryAt = int.MaxValue;
		private int startingFlags;
		private int flagsCaptured = 0;

		#endregion
		#region Constructors

		/// <summary>
		/// Creates a new ROMServer.
		/// </summary>
		public ROMServer(TreeFile file, Serializer ser)
			: base(file, ser)
		{
			aiNode = file.FindAtPath("AI");
		}

		#endregion
		#region Public Methods

		/// <summary>
		/// Starts the _game.
		/// </summary>
		public override void StartGame(IServerGame game, INode modeSettingsNode)
		{
			base.StartGame(game, modeSettingsNode);

			// Create AI player on each team 2 spawn point.
			foreach(IDoodad d in game.Doodads)
			{
				if(d.Rules.SpawnPoint && d.Rules.SpawnTeam == 2)
				{
					int type = getNewTankType(0, 2, true);
					TankPos pos = new TankPos(
						d.Location,
						Vector.Zero,
						chooseBestSpawnDirection(d.Location, 2, type),
						Angle.Zero,
						d.Location);
					int tank = game.SpawnNewTank(type, game.Owner, Color.Red, 0, 2, pos);
					IBot bot = new CustomBehaviorBot(game, aiNode, ser);
					game.AttachAIBot(tank, bot);
				}
			}

			// Count number of flags.
			startingFlags = 0;
			foreach(IDoodad d in game.Doodads)
			{
				if(d.Rules == game.Rules.Doodads[StringHasher.Instance.GetPortableHash("CTFFlag2")])
					startingFlags++;
			}
		}

		/// <summary>
		/// Spawns the human player.
		/// </summary>
		public override void SpawnNewPlayer(int owner, string name, System.Drawing.Color color, int player, int team, int preferredTank)
		{
			base.SpawnNewPlayer(owner, name, color, player, 1, preferredTank);
		}

		/// <summary>
		/// Performs mode-specific updates.
		/// </summary>
		public override void Update()
		{
			base.Update();

			// End _game?
			/*if(_game.Frame - 60 > deadAt)
				_game.EndGame(GameEndCause.Mode, strings.Defeat);*/
			if(game.Frame - 60 > victoryAt)
				game.EndGame(GameEndCause.ScenarioCompleted, 0);
		}

		/*/// <summary>
		/// Called when a tank is killed.
		/// </summary>
		public override void OnTankKilled(ITank tank, bool killedByZero)
		{
			// Game over if player has died.
			if(tank.Team == 1)
				deadAt = _game.Frame;
		}*/

		#endregion
		#region Non-Public Methods

		/// <summary>
		/// Called when a flag is returned.
		/// </summary>
		protected override void onFlagDelivered(ITank tank, List<Vector> originalFlags, int flagKey, int returnKey)
		{
			flagsCaptured++;

			// Victory?
			if(flagsCaptured >= startingFlags)
				victoryAt = game.Frame;
		}

		#endregion
	}
}